@@ -7,6 +7,10 @@
 #include <tcl.h>
 #include <Xm/Xm.h>
 
+#if USE_UIL
+#include <Mrm/MrmPublic.h>
+#endif
+
 #if XmVersion == 1001
 #  define MOTIF11
 #endif
@@ -50,6 +54,9 @@ typedef struct Tm_Display {
     Widget	commWidget;	/* used for send command */
     Atom 	registryProperty;
     Atom	commProperty;
+#if USE_UIL
+    MrmHierarchy hierarchy;
+#endif
 } Tm_Display;
 
 /*
@@ -108,6 +115,26 @@ typedef struct Tm_FreeResourceType {
     Tm_FreeProc free;
 } Tm_FreeResourceType;
 
+
+/*
+ * The following structure defines all of the commands supported by
+ * Tm, and the C procedures that execute them.
+ */
+
+typedef struct Tm_Cmd {
+    char *name;			/* Name of command. */
+    int (*cmdProc) _ANSI_ARGS_((ClientData clientData, Tcl_Interp *interp,
+	    int argc, char **argv));
+				/* Command procedure. */
+    int (*widgetCmdProc) _ANSI_ARGS_((ClientData clientData, Tcl_Interp *interp,
+	    int argc, char **argv));
+				/* Widget Command procedure. */
+} Tm_Cmd;
+
+extern Tm_Cmd Tm_Commands[];       /* the commands for Motif widgets */
+extern Tm_Cmd Tm_ExternCommands[]; /* the commands for extension widgets */
+
+
 /* strings for use in widget create commands */
 #define xmArrowButton		"xmArrowButton"
 #define xmBulletinBoard		"xmBulletinBoard"
@@ -153,4 +180,34 @@ typedef struct Tm_FreeResourceType {
 #define xmWarningDialog		"xmWarningDialog"
 #define xmWorkingDialog		"xmWorkingDialog"
 
+#if XmVERSION >= 2
+#define xmComboBox		"xmComboBox"
+#define xmContainer		"xmContainer"
+#define xmCSText		"xmCSText"
+#define xmDropDownComboBox	"xmDropDownComboBox"
+#define xmDropDownList		"xmDropDownList"
+#define xmIconGadget		"xmIconGadget"
+#define xmNotebook		"xmNotebook"
+#define xmSpinBox		"xmSpinBox"
+#endif
+
+/* this is used to link the string used in widget creation to the
+ * widget class
+ */
+typedef struct Tm_CommandToClassType {
+    char *command;
+    WidgetClass *clas;
+} Tm_CommandToClassType;
+
+extern Tm_CommandToClassType Tm_CommToClass[];		/* Motif widgets */
+extern Tm_CommandToClassType Tm_ExternCommandToClass[];	/* external widgets */
+
+/* mapping between reasons in callback call_data and the strings
+   that tclMotif would return in %reason substitutions
+ */
+typedef struct Tm_ReasonType {
+    int reason;
+    char *str;
+} Tm_ReasonType;
+
 #endif /* _TM_H */
