@@ -33,8 +33,62 @@ static char rcsid[] = "$Header";
 #include <tclXtSend.h>
 #include "tmFuncs.h"
 
+#include <X11/IntrinsicP.h>
+#include <X11/CoreP.h>  /* for access to class_name in Core class rec */
 
 
+#ifndef MOTIF11
+#include <Xm/RepType.h>
+#endif
+
+
+#ifndef MOTIF11
+#include <Xm/XmAll.h>
+#else
+#include <Xm/ArrowB.h>
+#include <Xm/BulletinB.h>
+#include <Xm/CascadeB.h>
+#include <Xm/Command.h>
+#include <Xm/DialogS.h>
+#include <Xm/DrawingA.h>
+#include <Xm/DrawnB.h>
+#include <Xm/FileSB.h>
+#include <Xm/Form.h>
+#include <Xm/Frame.h>
+#include <Xm/Label.h>
+#include <Xm/List.h>
+#include <Xm/MainW.h>
+#include <Xm/MessageB.h>
+#include <Xm/PanedW.h>
+#include <Xm/PushB.h>
+#include <Xm/RowColumn.h>
+#include <Xm/Scale.h>
+#include <Xm/ScrollBar.h>
+#include <Xm/ScrolledW.h>
+#include <Xm/SelectioB.h>
+#include <Xm/Separator.h>
+#include <Xm/Text.h>
+#include <Xm/TextF.h>
+#include <Xm/ToggleB.h>
+#endif  /* MOTIF11 */
+
+#if USE_UIL
+#include <Mrm/MrmPublic.h>
+
+extern void
+Tm_UILCallbackHandler _ANSI_ARGS_((Widget,
+	XtPointer, XtPointer));
+
+static MrmRegisterArg
+uil_callbacks[] = {
+    {"tcl", (XtPointer) Tm_UILCallbackHandler}
+};
+#endif
+
+#include <X11/Shell.h>
+
+extern Tm_CommandToClassType Tm_CommToClass[];
+
 /*
  *----------------------------------------------------------------------
  * Tm_Class -
@@ -273,12 +327,30 @@ Tm_AppInitialize(clientData, interp, argc, argv)
 	    if (strcmp(argv[n], "-fallbackResources") == 0) {
 		Tcl_SplitList(interp, argv[n+1], &num_fallback_resources,
 				&fallback_resources);
+		/* the fallback list must be NULL terminated, so add a NULL */
+		fallback_resources = 
+			(char **) XtRealloc((char *) fallback_resources,
+				(num_fallback_resources + 1) * 
+					sizeof(char *));
+		fallback_resources[num_fallback_resources + 1] = NULL;
 		n++;
 		continue;
 	    }
 	}
     }
 
+#   ifndef MOTIF11
+        XtSetLanguageProc(NULL, NULL, NULL);
+#   endif
+
+#   if USE_UIL
+	MrmInitialize();
+	MrmRegisterNames(uil_callbacks, XtNumber(uil_callbacks));
+        Tcl_SetVar(interp, "_Tm_UIL", "1", TCL_GLOBAL_ONLY);
+#else
+        Tcl_SetVar(interp, "_Tm_UIL", "0", TCL_GLOBAL_ONLY);
+#   endif
+
     /* class set, or derive it from argv0? */
     if (Class == NULL) 
 	Class = Tm_Class(interp);
@@ -334,10 +406,18 @@ Tm_AppInitialize(clientData, interp, argc, argv)
 
     free((char *) app_argv);
 
+    /* register the tear-off menu converter */
+#   ifndef MOTIF11
+	XmRepTypeInstallTearOffModelConverter();
+#   endif
+
 #   ifdef MALLOC_TRACE
 	mal_leaktrace(1);
 #   endif
 
+    /* perform any startup stuff for any extension widgets */
+    Tm_ExternWidgetsInitialise(interp);
+
     return TCL_OK;
 }
 
@@ -366,6 +446,10 @@ Tm_Init (interp)
 {
     char *libDir;
     char buf[32];
+    static char initCmd[] =
+        "if [file exists $tm_library/init_tclMotif.tcl] {\n\
+            source  $tm_library/init_tclMotif.tcl\n\
+        }";
 
     /*
      * Bind in Tm's commands.
@@ -397,5 +481,10 @@ Tm_Init (interp)
     sprintf(buf, "%d", XmREVISION);
     Tcl_SetVar(interp, "XmREVISION", buf, TCL_GLOBAL_ONLY);
 
-    return TCL_OK;
+    /* create the command to extract info from an X event */
+    Tcl_CreateCommand(interp, "xEvent", Tm_XEvent,
+                        NULL, NULL);
+
+    /* load library */
+    return Tcl_Eval(interp, initCmd);
 }
