@@ -28,6 +28,10 @@ static char rcsid[] = "$Header";
 
 #include "tm.h"
 #include "tmFuncs.h"
+
+#include <X11/IntrinsicP.h>
+#include <X11/CoreP.h>	/* for access to class_name in Core class rec */
+
 #ifndef MOTIF11
 #include <Xm/DragDrop.h>
 #endif
@@ -35,6 +39,13 @@ static char rcsid[] = "$Header";
 #include <Xm/List.h>
 #include <Xm/Text.h>
 #include <Xm/RowColumn.h>
+#if XmVERSION >= 2
+#include <Xm/CSText.h>
+#endif
+
+#if USE_UIL
+#include <Mrm/MrmPublic.h>
+#endif
 		
 /*--------------------------------------------------------------
  *
@@ -69,7 +80,7 @@ Tm_AnyWidgetCmd(clientData, interp, argc, argv)
 
     if (argc < 2) {
 	sprintf(interp->result,
-		"wrong # args: should be \"%.50s option [arg arg ...]\"",
+		"wrong # args: should be \"%.50s method [arg arg ...]\"",
 		argv[0]);
 	return TCL_ERROR;
     }
@@ -108,6 +119,15 @@ action", argv[0]);
 	return TCL_OK;
     } else
 
+    if ((c == 'c') && (strcmp(argv[1], "class") == 0)) {
+	CoreWidgetClass c;
+
+	c = (CoreWidgetClass) XtClass(wPtr->widget);
+	Tcl_SetResult(interp, c->core_class.class_name, TCL_STATIC);
+
+	return TCL_OK;
+    } else
+
     if ((c == 'd') && (strcmp(argv[1], "defineCursor") == 0)) {
 	XrmValue from, to;
 	Cursor cursor;
@@ -167,8 +187,9 @@ value", argv[0]);
 	Tm_HackDragWidgetPtr = wPtr;
 
 	w = XmDragStart(wPtr->widget, Tm_HackXEvent, NULL, 0);
-	Tm_SetValues(wPtr->pathName, interp, w, w, XtClass(w),
-				argv+2, argc-2, args, &num_args);
+	if (Tm_SetValues(wPtr->pathName, interp, w, w, XtClass(w),
+		argv+2, argc-2, args, &num_args) == TCL_ERROR)
+	    return TCL_ERROR;
 	XtSetValues(w, args, num_args);
  
     } else
@@ -206,7 +227,7 @@ resourceList\"", argv[0]);
 
     if ((c == 'g') && (strcmp(argv[1], "getValues") == 0)) {
 	if (argc < 4) {
-	    sprintf(interp->result, "wrong # args: should be \"%.50s setValues \
+	    sprintf(interp->result, "wrong # args: should be \"%.50s getValues \
 -option value...", argv[0]);
 	    return TCL_ERROR;
 	}
@@ -266,6 +287,44 @@ resourceList\"", argv[0]);
 	XtMapWidget(wPtr->widget);
     } else
 
+#if USE_UIL
+    if ((c == 'm') && (strcmp(argv[1], "mrmFetchWidget") == 0)) {
+	Widget wid;
+	MrmType cl;
+	char *path;
+
+	if (argc < 3) {
+	    sprintf(interp->result, "wrong # args: should be \"%.50s mrmFetchWidget \
+widget", argv[0]);
+	    return TCL_ERROR;
+	}
+
+	if (MrmFetchWidget(wPtr->displayInfo->hierarchy, argv[2],
+		w, &wid, &cl) != MrmSUCCESS) {
+	    sprintf(interp->result, "failed to fetch widget %s",
+		argv[2]);
+	    return TCL_ERROR;
+	}
+
+	/* make this and its children into Tm widgets */
+	path = XtMalloc(strlen(argv[0]) + strlen(argv[2]) + 2);
+	strcpy(path, argv[0]);
+	/* special case root widget `.' */
+	if (strcmp(argv[0], ".") != 0) {
+	    strcat(path, ".");
+	}
+	strcat(path, argv[2]);
+	Tm_CreateTmInfo(interp, wid, path, argv[0], wPtr->displayInfo);
+	if (XtIsComposite(wid)) {
+	    Tm_CreateTmInfoInChildren(interp, wid, path,
+			wPtr->displayInfo);
+	}
+	XtFree(path);
+	
+	return TCL_OK;
+    } else
+#endif /* USE_UIL */
+
     if ((c == 'p') && (strcmp(argv[1], "parent") == 0)) {
 	Tcl_SetResult(interp, XtNewString(wPtr->parent), 
 			TCL_DYNAMIC);
@@ -332,7 +391,7 @@ direction", argv[0]);
 	/* set Sensitive */
 	if (argc < 3) {
 	    sprintf(interp->result, "wrong # args: should be \"%.50s setSensitive \
--option value", argv[0]);
+value", argv[0]);
 	    return TCL_ERROR;
 	}
 	if (Tcl_GetBoolean(interp, argv[2], &bool_val) == TCL_ERROR) {
@@ -375,13 +434,114 @@ direction", argv[0]);
 	XtUnmapWidget(wPtr->widget);
     } else
 
+    /* all callback methods are of the form *Callback
+       (well, some Wcs ones aren't, but I dont support them)
+     */
+    if ((c == 'r') && (strcmp(argv[1], "removeCallback") == 0)) {
+	XtCallbackList callback_list;
+	char *callback;
+	char *callback_func;
+	Tm_ClientData *client_data;
+
+        if (argc < 4) {
+            sprintf(interp->result, "wrong # args: should be \"%.50s removeCallback \
+callback command...", argv[0]);
+            return TCL_ERROR;
+        }
+
+	/* we can only get the callback list from the widget */
+	callback = argv[2];
+	if (XtHasCallbacks(wPtr->widget, callback) != XtCallbackHasSome) {
+	    sprintf(interp->result, "%s: cant remove callback %s %s",
+			argv[0], argv[2], argv[3]);
+	    return TCL_ERROR;
+	}
+	XtVaGetValues(wPtr->widget, callback, &callback_list, NULL);
+
+	client_data = (Tm_ClientData *) XtMalloc(sizeof(Tm_ClientData));
+
+	if (argc == 4) {
+	    /* for backward compatability with eg argv[3] = {do it} */
+	    callback_func = XtNewString(argv[3]);
+	} else {
+	    callback_func = Tcl_Merge(argc - 3, argv + 3);
+	}
+
+	/* now look to see if we can find a match */
+	while (callback_list != NULL) {
+	    if (callback_list->callback == Tm_WidgetCallbackHandler) {
+		client_data = (Tm_ClientData *) (callback_list->closure);
+		if (strcmp(callback_func, client_data->callback_func)
+				== 0) {
+		    /* found an exact match, so remove it */
+		    XtRemoveCallback(wPtr->widget, argv[2], Tm_WidgetCallbackHandler,
+				(XtPointer) client_data);
+		    XtRemoveCallback(wPtr->widget, XmNdestroyCallback, 
+				Tm_DestroyReclaimHandler, (XtPointer) client_data);
+
+		    XtFree((char *) client_data);
+		    XtFree(callback_func);
+
+		    return TCL_OK;
+		}
+	    }
+	    callback_list++;
+	}
+
+	XtFree(callback_func);
+    } else
+
+#if XmVERSION >= 2
+    /* in Motif 2.0 the UTM functions need special handling
+     */
+    if ((c == 'd') && (strcmp(argv[1], "destinationCallback") == 0)) {
+        Tm_ClientData *client_data;
+
+        if (argc != 3) {
+            sprintf(interp->result,
+                "wrong # args: should be \"%.50s destinationCallback callback\"",
+                argv[0]);
+            return TCL_ERROR;
+        }
+        client_data = (Tm_ClientData *) XtMalloc(sizeof(Tm_ClientData));
+        client_data->callback_func = XtNewString(argv[2]);
+        client_data->widget_info = wPtr;
+
+        XtAddCallback(wPtr->widget, XmNdestinationCallback,
+                        Tm_DestinationCallbackHandler, (XtPointer) client_data);
+        XtAddCallback(wPtr->widget, XmNdestroyCallback, 
+                Tm_DestroyReclaimHandler, (XtPointer) client_data);
+    } else
+
+    if ((c == 'c') && (strcmp(argv[1], "convertCallback") == 0)) {
+        Tm_ClientData *client_data;
+
+        if (argc != 3) {
+            sprintf(interp->result,
+                "wrong # args: should be \"%.50s convertCallback callback\"",
+                argv[0]);
+            return TCL_ERROR;
+        }
+        client_data = (Tm_ClientData *) XtMalloc(sizeof(Tm_ClientData));
+        client_data->callback_func = XtNewString(argv[2]);
+        client_data->widget_info = wPtr;
+
+        XtAddCallback(wPtr->widget, XmNconvertCallback,
+                        Tm_ConvertCallbackHandler, (XtPointer) client_data);
+        XtAddCallback(wPtr->widget, XmNdestroyCallback, 
+                Tm_DestroyReclaimHandler, (XtPointer) client_data);
+
+    } else
+#endif /*  XmVERSION >= 2 */
+
+    /* all callback methods are of the form *Callback
+       (well, some Wcs ones aren't, but I dont support them)
+     */
     if (strstr(argv[1], "Callback") != NULL) {
 	Tm_ClientData *client_data;
 
 	client_data = (Tm_ClientData *) XtMalloc(sizeof(Tm_ClientData));
-/*
-	client_data->callback_func = XtNewString(argv[2]);
-*/
+
 	if (argc == 3) {
 	    /* for backward compatability with eg argv[2] = {do it} */
 	    client_data->callback_func = XtNewString(argv[2]);
@@ -399,7 +559,7 @@ direction", argv[0]);
     return TCL_OK;
 
     error:
-    sprintf(interp->result, "%s: unknown option \"%s\"", argv[0], argv[1]);
+    sprintf(interp->result, "%s: unknown method \"%s\"", argv[0], argv[1]);
     return TCL_ERROR;
 }
 
@@ -432,7 +592,7 @@ Tm_CommandWidgetCmd (clientData, interp, argc, argv)
 
     if (argc < 2) {
 	sprintf(interp->result,
-		"wrong # args: should be \"%.50s option [arg arg ...]\"",
+		"wrong # args: should be \"%.50s method [arg arg ...]\"",
 		argv[0]);
 	return TCL_ERROR;
     }
@@ -534,7 +694,7 @@ Tm_ListWidgetCmd (clientData, interp, argc, argv)
 
     if (argc < 2) {
 	sprintf(interp->result,
-		"wrong # args: should be \"%.50s option [arg arg ...]\"",
+		"wrong # args: should be \"%.50s method [arg arg ...]\"",
 		argv[0]);
 	return TCL_ERROR;
     }
@@ -915,7 +1075,7 @@ Tm_RootCmd (clientData, interp, argc, argv)
 
     if (argc < 2) {
 	sprintf(interp->result,
-		"wrong # args: should be \"%.50s option [arg arg ...]\"",
+		"wrong # args: should be \"%.50s method [arg arg ...]\"",
 		argv[0]);
 	return TCL_ERROR;
     }
@@ -1020,6 +1180,50 @@ permissions tcl_code", argv[0]);
 	return TCL_OK;
     } else
 
+    if ((c == 'm') && (strcmp(argv[1], "mainLoop") == 0)) {
+        XtAppMainLoop(XtWidgetToApplicationContext(wPtr->widget));
+    } else
+
+#if USE_UIL
+    if ((c == 'm') && (strcmp(argv[1], "mrmCloseHierarchy") == 0)) {
+	if (MrmCloseHierarchy(wPtr->displayInfo->hierarchy) != MrmSUCCESS) {
+	    sprintf(interp->result, "failed to close Mrm hierachy");
+	    return TCL_ERROR;
+	}
+	return TCL_OK;
+    } else
+#endif /* USE_UIL */
+
+    if ((c == 'm') && (strcmp(argv[1], "mrmOpenHierarchy") == 0)) {
+	int largc;
+	char **largv;
+#if USE_UIL
+	MrmHierarchy *hierarchy;
+
+        if (argc < 3) {
+            sprintf(interp->result,
+                        "wrong # args: should be \"%.50s mrmOpenHierarchy file_list ",
+                        argv[0]);
+            return TCL_ERROR;
+        }
+	
+	hierarchy = &(wPtr->displayInfo->hierarchy);
+
+	Tcl_SplitList(interp, argv[2], &largc, &largv);
+	if (MrmOpenHierarchyPerDisplay(XtDisplay(w), largc, largv, NULL, hierarchy) !=
+		MrmSUCCESS) {
+	    sprintf(interp->result, "failed to open Mrm hierarchy");
+	    return TCL_ERROR;
+	}
+	free(argv);
+	return TCL_OK;
+#else
+	sprintf(interp->result, "This version of moat not compiled with UIL support");
+	return TCL_ERROR;
+#endif /* USE_UIL */
+
+    } else
+
     if ((c == 'p') && (strcmp(argv[1], "processEvent") == 0)) {
 	XtAppProcessEvent(appContext, XtIMAll);
     } else
@@ -1058,10 +1262,6 @@ timerID\"", argv[0]);
 	    return TCL_ERROR;
 	}
 	XtRemoveTimeOut(timerId);
-    } else
-
-    if ((c == 'm') && (strcmp(argv[1], "mainLoop") == 0)) {
-	XtAppMainLoop(XtWidgetToApplicationContext(wPtr->widget));
 
     } else {
         return Tm_ShellWidgetCmd(clientData, interp, argc, argv);
@@ -1098,7 +1298,7 @@ Tm_DrawnWidgetCmd (clientData, interp, argc, argv)
 
     if (argc < 2) {
 	sprintf(interp->result,
-		"wrong # args: should be \"%.50s option [arg arg ...]\"",
+		"wrong # args: should be \"%.50s method [arg arg ...]\"",
 		argv[0]);
 	return TCL_ERROR;
     }
@@ -1244,7 +1444,7 @@ Tm_TextWidgetCmd (clientData, interp, argc, argv)
 
     if (argc < 2) {
 	sprintf(interp->result,
-		"wrong # args: should be \"%.50s option [arg arg ...]\"",
+		"wrong # args: should be \"%.50s method [arg arg ...]\"",
 		argv[0]);
 	return TCL_ERROR;
     }
@@ -1777,6 +1977,644 @@ Tm_TextWidgetCmd (clientData, interp, argc, argv)
     return TCL_OK;
 }
 
+
+#if XmVERSION >= 2
+/*
+ *--------------------------------------------------------------
+ *
+ * Tm_CSTextWidgetCmd --
+ *
+ *	handles the methods that may be requested of CSText widgets.
+ *
+ * Results:
+ *
+ * Side effects:
+ *
+ *--------------------------------------------------------------
+ */
+
+int
+Tm_CSTextWidgetCmd (clientData, interp, argc, argv)
+    ClientData clientData;
+    Tcl_Interp *interp;
+    int argc;
+    char **argv;
+{
+    Tm_Widget *wPtr = (Tm_Widget *) clientData;
+    Widget w = wPtr->widget;
+    Display *display;
+    Time timeStamp;
+
+    if (argc < 2) {
+	sprintf(interp->result,
+		"wrong # args: should be \"%.50s method [arg arg ...]\"",
+		argv[0]);
+	return TCL_ERROR;
+    }
+    
+    display = XtDisplay(wPtr->widget);
+    timeStamp = XtLastTimestampProcessed(display);
+
+    if (strcmp(argv[1], "losingFocusCallback") == 0) {
+        Tm_ClientData *client_data;
+
+        if (argc != 3) {
+            sprintf(interp->result,
+                "wrong # args: should be \"%.50s losingFocusCallback callback\"",
+                argv[0]);
+            return TCL_ERROR;
+        }
+        client_data = (Tm_ClientData *) XtMalloc(sizeof(Tm_ClientData));
+        client_data->callback_func = XtNewString(argv[2]);
+        client_data->widget_info = wPtr;
+
+	XtAddCallback(wPtr->widget, XmNlosingFocusCallback,
+			Tm_CSTextVerifyCallbackHandler,
+			(XtPointer) client_data);
+        XtAddCallback(wPtr->widget, XmNdestroyCallback, 
+                Tm_DestroyReclaimHandler, (XtPointer) client_data);
+    } else
+
+    if (strcmp(argv[1], "modifyVerifyCallback") == 0) {
+        Tm_ClientData *client_data;
+
+        if (argc != 3) {
+            sprintf(interp->result,
+                "wrong # args: should be \"%.50s modifyVerifyCallback callback\"",
+                argv[0]);
+            return TCL_ERROR;
+        }
+        client_data = (Tm_ClientData *) XtMalloc(sizeof(Tm_ClientData));
+        client_data->callback_func = XtNewString(argv[2]);
+        client_data->widget_info = wPtr;
+
+	XtAddCallback(wPtr->widget, XmNmodifyVerifyCallback,
+			Tm_CSTextVerifyCallbackHandler,
+			(XtPointer) client_data);
+        XtAddCallback(wPtr->widget, XmNdestroyCallback, 
+                Tm_DestroyReclaimHandler, (XtPointer) client_data);
+    } else
+
+    if (strcmp(argv[1], "motionVerifyCallback") == 0) {
+        Tm_ClientData *client_data;
+
+        if (argc != 3) {
+            sprintf(interp->result,
+                "wrong # args: should be \"%.50s motionVerifyCallback callback\"",
+                argv[0]);
+            return TCL_ERROR;
+        }
+        client_data = (Tm_ClientData *) XtMalloc(sizeof(Tm_ClientData));
+        client_data->callback_func = XtNewString(argv[2]);
+        client_data->widget_info = wPtr;
+
+	XtAddCallback(wPtr->widget, XmNmotionVerifyCallback,
+			Tm_CSTextVerifyCallbackHandler,
+			(XtPointer) client_data);
+        XtAddCallback(wPtr->widget, XmNdestroyCallback, 
+                Tm_DestroyReclaimHandler, (XtPointer) client_data);
+    } else
+
+    if (strcmp(argv[1], "clearSelection") == 0) {
+	XmCSTextClearSelection(wPtr->widget, timeStamp);
+    } else
+
+    if (strcmp(argv[1], "copy") == 0) {
+	XmCSTextCopy(wPtr->widget, timeStamp);
+    } else
+
+    if (strcmp(argv[1], "cut") == 0) {
+	XmCSTextCut(wPtr->widget, timeStamp);
+    } else
+
+    if (strcmp(argv[1], "disableRedisplay") == 0) {
+	XmCSTextDisableRedisplay(w);
+    } else
+
+    if (strcmp(argv[1], "enableRedisplay") == 0) {
+	XmCSTextEnableRedisplay(w);
+    } else
+
+    if (strcmp(argv[1], "findString") == 0) {
+	long start;
+	char *string;
+	XmTextDirection direction;
+	long position;
+	char pos_str[16];
+
+        if (argc != 6) {
+            sprintf(interp->result,
+                "wrong # args: should be \"%.50s findString start string direction position\"",
+                argv[0]);
+            return TCL_ERROR;
+        }
+	if (sscanf(argv[2], "%ld", &start) < 1) {
+	    sprintf(interp->result, "non-integer start param for %.50s findString",
+		argv[0]);
+	    return TCL_ERROR;
+	}
+	string = argv[3];
+	if (strcmp(argv[4], "forward") == 0) {
+	    direction = XmTEXT_FORWARD;
+	} else
+	if (strcmp(argv[4], "backward") == 0) {
+	    direction = XmTEXT_BACKWARD;
+	} else {
+	    sprintf(interp->result, "%.50s findString: direction must be forward or backward",
+		argv[0]);
+	    return TCL_ERROR;
+	}
+	
+	if (XmCSTextFindString(w, start, string, direction, &position)) {
+	    sprintf(interp->result, "true");
+	    sprintf(pos_str, "%ld", position);
+	    if (Tcl_SetVar(interp, argv[5], pos_str, 0) == NULL) {
+		return TCL_ERROR;
+	    } else
+		return TCL_OK;
+	} else {
+	    sprintf(interp->result, "false");
+	    return TCL_OK;
+	}
+    } else
+
+    if (strcmp(argv[1], "getEditable") == 0) {
+	if (XmCSTextGetEditable(w)) {
+	    interp->result = "true";
+	} else {
+	    interp->result = "false";
+	}
+	return TCL_OK;
+    } else
+
+    if (strcmp(argv[1], "getInsertionPosition") == 0) {
+	XmTextPosition pos;
+
+	pos = XmCSTextGetInsertionPosition(w);
+	sprintf(interp->result, "%ld", pos);
+	return TCL_OK;
+    } else
+
+    if (strcmp(argv[1], "getLastPosition") == 0) {
+	XmTextPosition pos;
+
+	pos = XmCSTextGetLastPosition(w);
+	sprintf(interp->result, "%ld", pos);
+	return TCL_OK;
+    } else
+
+    if (strcmp(argv[1], "getSelection") == 0) {
+	XmString sel;
+	char *str;
+
+	sel = XmCSTextGetSelection(w);
+	str = Tm_XmStringToString(sel);
+	Tcl_SetResult(interp, str, TCL_VOLATILE);
+	XtFree(str);
+	XmStringFree(sel);
+	return TCL_OK;
+    } else
+
+    if (strcmp(argv[1], "getSelectionPosition") == 0) {
+	XmTextPosition left, right;
+	char buf[16];
+
+        if (argc != 4) {
+            sprintf(interp->result,
+                "wrong # args: should be \"%.50s getSelectionPosition left right\"",
+                argv[0]);
+            return TCL_ERROR;
+        }
+	if (XmCSTextGetSelectionPosition(w, &left, &right)) {
+	    interp->result = "true";
+	} else {
+	    interp->result = "false";
+	}
+	sprintf(buf, "%ld", left);
+	Tcl_SetVar(interp, argv[2], buf, 0);
+	sprintf(buf, "%ld", right);
+	Tcl_SetVar(interp, argv[3], buf, 0);
+
+	return TCL_OK;
+    } else
+
+    if (strcmp(argv[1], "getString") == 0) {
+	XmString xmstr;
+	char *str;
+
+	xmstr = XmCSTextGetString(wPtr->widget);
+	str = Tm_XmStringToString(xmstr);
+	Tcl_SetResult(interp, str, TCL_VOLATILE);
+	XtFree(str);
+	XmStringFree(xmstr);
+    } else
+
+    if (strcmp(argv[1], "getSubstring") == 0) {
+	XmTextPosition start;
+	int num_chars;
+	int buffer_size;
+	char *buffer;
+	int ret;
+
+        if (argc != 5) {
+            sprintf(interp->result,
+                "wrong # args: should be \"%.50s getSubstring start num_chars buffer\"",
+                argv[0]);
+            return TCL_ERROR;
+        }
+
+	if (sscanf(argv[2], "%ld", &start) < 1) {
+	    sprintf(interp->result, "%.50s getSubstring: bad num_chars",
+		argv[0]);
+	    return TCL_ERROR;
+	}
+	if (sscanf(argv[3], "%ld", &num_chars) < 1) {
+	    sprintf(interp->result, "%.50s getSubstring: bad num_chars",
+		argv[0]);
+	    return TCL_ERROR;
+	}
+
+	/* MB_CUR_MAX defaults to zero on Sun */
+	if (MB_CUR_MAX >= 1) {
+	    buffer_size = num_chars * MB_CUR_MAX + 1;
+	} else {
+	    buffer_size = num_chars + 1;
+	}
+	buffer = XtMalloc(buffer_size);
+	
+	ret = XmCSTextGetSubstring(w, start, num_chars, buffer_size, buffer);
+	Tcl_SetVar(interp, argv[4], buffer, 0);
+
+	if ( ret == XmCOPY_SUCCEEDED) {
+	    interp->result = "succeeded";
+	} else
+	if (ret == XmCOPY_TRUNCATED) {
+	    interp->result = "truncated";
+	} else
+	if (ret == XmCOPY_FAILED) {
+	    interp->result = "failed";
+	}
+
+	XtFree(buffer);
+	return TCL_OK;
+    } else
+
+    if (strcmp(argv[1], "getTopCharacter") == 0) {
+	XmTextPosition pos;
+
+	pos = XmCSTextGetTopCharacter(w);
+	sprintf(interp->result, "%ld", pos);
+	return TCL_OK;
+    } else
+
+    if (strcmp(argv[1], "insert") == 0) {
+	XmTextPosition pos;
+	XmString xmstr;
+
+        if (argc != 4) {
+            sprintf(interp->result,
+                "wrong # args: should be \"%.50s insert position value\"",
+                argv[0]);
+            return TCL_ERROR;
+        }
+	
+	if (sscanf(argv[2], "%ld", &pos) < 1) {
+	    sprintf(interp->result, "%.50s insert: bad position", argv[0]);
+	    return TCL_ERROR;
+	}
+	
+	xmstr = Tm_StringToXmString(interp, argv[3]);
+	XmCSTextInsert(w, pos, xmstr);
+	XmStringFree(xmstr);
+
+	return TCL_OK;
+    } else
+
+    if (strcmp(argv[1], "paste") == 0) {
+	XmCSTextPaste(wPtr->widget);
+    } else
+
+    if (strcmp(argv[1], "remove") == 0) {
+	XmCSTextRemove(wPtr->widget);
+    } else
+
+    if (strcmp(argv[1], "replace") == 0) {
+	XmTextPosition from, to;
+	XmString xmstr;
+
+        if (argc != 5) {
+            sprintf(interp->result,
+                "wrong # args: should be \"%.50s replace from to value\"",
+                argv[0]);
+            return TCL_ERROR;
+        }
+
+	if (sscanf(argv[2], "%ld", &from) < 1) {
+	    sprintf(interp->result, "%.50s replace: bad from value", argv[0]);
+	    return TCL_ERROR;
+	}
+	if (sscanf(argv[3], "%ld", &to) < 1) {
+	    sprintf(interp->result, "%.50s replace: bad to value", argv[0]);
+	    return TCL_ERROR;
+	}
+	
+	xmstr = Tm_StringToXmString(interp, argv[4]);
+	XmCSTextReplace(w, from, to, xmstr);
+	XmStringFree(xmstr);
+	return TCL_OK;
+    } else
+
+    if (strcmp(argv[1], "scroll") == 0) {
+	int lines;
+
+        if (argc != 3) {
+            sprintf(interp->result,
+                "wrong # args: should be \"%.50s scroll lines\"",
+                argv[0]);
+	    return TCL_ERROR;
+	}
+    
+	if (Tcl_GetInt(interp, argv[2], &lines) != TCL_OK) {
+	    return TCL_ERROR;
+	}
+	XmCSTextScroll(w, lines);
+        return TCL_OK;
+    } else
+
+    if (strcmp(argv[1], "setAddMode") == 0) {
+	int state;
+
+        if (argc != 3) {
+            sprintf(interp->result,
+                "wrong # args: should be \"%.50s setAddMode boolean\"",
+                argv[0]);
+	    return TCL_ERROR;
+	}
+
+	if (Tcl_GetBoolean(interp, argv[2], &state) != TCL_OK) {
+	    return TCL_ERROR;
+	}
+
+	XmCSTextSetAddMode(w, state);
+	return TCL_OK;
+    } else
+
+    if (strcmp(argv[1], "setEditable") == 0) {
+	int editable;
+
+        if (argc != 3) {
+            sprintf(interp->result,
+                "wrong # args: should be \"%.50s setEditable boolean\"",
+                argv[0]);
+	    return TCL_ERROR;
+	}
+
+	if (Tcl_GetBoolean(interp, argv[2], &editable) != TCL_OK) {
+	    return TCL_ERROR;
+	}
+
+	XmCSTextSetEditable(w, editable);
+	return TCL_OK;
+    } else
+
+    if (strcmp(argv[1], "setHighlight") == 0) {
+	XmTextPosition left, right;
+	XmHighlightMode mode;
+
+        if (argc != 5) {
+            sprintf(interp->result,
+                "wrong # args: should be \"%.50s setHighlight left right mode\"",
+                argv[0]);
+	    return TCL_ERROR;
+	}
+
+	if (sscanf(argv[2], "%ld", &left) < 1) {
+	    sprintf(interp->result, "%.50s setHighlight: bad \"left\" value", argv[0]);
+	    return TCL_ERROR;
+	}
+	if (sscanf(argv[3], "%ld", &right) < 1) {
+	    sprintf(interp->result, "%.50s setHighlight: bad \"right\" value", argv[0]);
+	    return TCL_ERROR;
+	}
+
+	if (strcmp(argv[4], "normal") == 0) {
+	    mode = XmHIGHLIGHT_NORMAL;
+	} else
+	if (strcmp(argv[4], "selected") == 0) {
+	    mode = XmHIGHLIGHT_SELECTED;
+	} else
+	if (strcmp(argv[4], "secondary_selected") == 0) {
+	    mode = XmHIGHLIGHT_SECONDARY_SELECTED;
+	} else {
+	    sprintf(interp->result, "%.50s setHighlight: bad \"highlight\" value", argv[0]);
+	    return TCL_ERROR;
+	}
+	    
+	XmCSTextSetHighlight(w, left, right, mode);
+	return TCL_OK;
+    } else
+
+    if (strcmp(argv[1], "setInsertionPosition") == 0) {
+	XmTextPosition pos;
+
+        if (argc != 3) {
+            sprintf(interp->result,
+                "wrong # args: should be \"%.50s setInsertionPosition position\"",
+                argv[0]);
+            return TCL_ERROR;
+        }
+
+	if (sscanf(argv[2], "%ld", &pos) < 1) {
+	    sprintf(interp->result, "%.50s setInsertionPosition: bad \"position\" value", argv[0]);
+	    return TCL_ERROR;
+	}
+
+	XmCSTextSetInsertionPosition(w, pos);
+	return TCL_OK;
+    } else
+
+    if (strcmp(argv[1], "setSelection") == 0) {
+	XmTextPosition first, last;
+	Time tim;
+
+        if (argc != 4) {
+            sprintf(interp->result,
+                "wrong # args: should be \"%.50s setSelection first last\"",
+                argv[0]);
+            return TCL_ERROR;
+        }
+
+	if (sscanf(argv[2], "%ld", &first) < 1) {
+	    sprintf(interp->result, "%.50s setSelection: bad \"first\" value", argv[0]);
+	    return TCL_ERROR;
+	}
+	if (sscanf(argv[3], "%ld", &last) < 1) {
+	    sprintf(interp->result, "%.50s setSelection: bad \"last\" value", argv[0]);
+	    return TCL_ERROR;
+	}
+
+	tim = XtLastTimestampProcessed(XtDisplay(w));
+
+	XmCSTextSetSelection(w, first, last, tim);
+	return TCL_OK;
+    } else
+
+    if (strcmp(argv[1], "setSource") == 0) {
+	XmTextPosition top, position;
+	XmCSTextSource source;
+	Widget w_old;
+	Tm_Widget *info;
+
+        if (argc != 5) {
+            sprintf(interp->result,
+                "wrong # args: should be \"%.50s setSource source top position\"",
+                argv[0]);
+            return TCL_ERROR;
+        }
+
+	info = Tm_WidgetInfoFromPath(interp, argv[2]);
+	if (info == NULL) {
+	    sprintf(interp->result, "%.50s setSource: bad \"widget\" value", argv[0]);
+	    return TCL_ERROR;
+	}
+	w_old = info->widget;
+	source = XmCSTextGetSource(w_old);
+	    
+	if (sscanf(argv[3], "%ld", &top) < 1) {
+	    sprintf(interp->result, "%.50s setSource: bad \"top\" value", argv[0]);
+	    return TCL_ERROR;
+	}
+	if (sscanf(argv[4], "%ld", &position) < 1) {
+	    sprintf(interp->result, "%.50s setSource: bad \"position\" value", argv[0]);
+	    return TCL_ERROR;
+	}
+
+	XmCSTextSetSource(w, source, top, position);
+	return TCL_OK;
+    } else
+
+    if (strcmp(argv[1], "setString") == 0) {
+	XmString xmstr;
+
+        if (argc != 3) {
+            sprintf(interp->result,
+                "wrong # args: should be \"%.50s setString value\"",
+                argv[0]);
+            return TCL_ERROR;
+        }
+	
+	xmstr = Tm_StringToXmString(interp, argv[2]);
+	XmCSTextSetString(w, xmstr);
+	XmStringFree(xmstr);
+	return TCL_OK;
+    } else
+
+    if (strcmp(argv[1], "setTopCharacter") == 0) {
+	XmTextPosition top;
+
+        if (argc != 3) {
+            sprintf(interp->result,
+                "wrong # args: should be \"%.50s setTopCharacter top\"",
+                argv[0]);
+            return TCL_ERROR;
+        }
+
+	if (sscanf(argv[2], "%ld", &top) < 1) {
+	    sprintf(interp->result, "%.50s setTopCharacter: bad \"top\" value", argv[0]);
+	    return TCL_ERROR;
+	}
+
+	XmCSTextSetTopCharacter(w, top);
+	return TCL_OK;
+    } else
+
+    if (strcmp(argv[1], "showPosition") == 0) {
+	XmTextPosition pos;
+
+        if (argc != 3) {
+            sprintf(interp->result,
+                "wrong # args: should be \"%.50s showPosition position\"",
+                argv[0]);
+            return TCL_ERROR;
+        }
+
+	if (sscanf(argv[2], "%ld", &pos) < 1) {
+	    sprintf(interp->result, "%.50s showPosition: bad \"position\" value", argv[0]);
+	    return TCL_ERROR;
+	}
+
+	XmCSTextShowPosition(w, pos);
+	return TCL_OK;
+    } else {
+        return Tm_AnyWidgetCmd(clientData, interp, argc, argv);
+    }
+    return TCL_OK;
+}
+#endif /* XmVERSION >= 2 */
+
+
+#if XmVERSION >= 2
+/*
+ *--------------------------------------------------------------
+ *
+ * Tm_NotebookWidgetCmd --
+ *
+ *	handles the methods that may be requested of Text widgets.
+ *
+ * Results:
+ *
+ * Side effects:
+ *
+ *--------------------------------------------------------------
+ */
+
+int
+Tm_NotebookWidgetCmd (clientData, interp, argc, argv)
+    ClientData clientData;
+    Tcl_Interp *interp;
+    int argc;
+    char **argv;
+{
+    Tm_Widget *wPtr = (Tm_Widget *) clientData;
+    Widget w = wPtr->widget;
+    Display *display;
+    Time timeStamp;
+
+    if (argc < 2) {
+	sprintf(interp->result,
+		"wrong # args: should be \"%.50s method [arg arg ...]\"",
+		argv[0]);
+	return TCL_ERROR;
+    }
+    
+    display = XtDisplay(wPtr->widget);
+    timeStamp = XtLastTimestampProcessed(display);
+
+    if (strcmp(argv[1], "modifyVerifyCallback") == 0) {
+        Tm_ClientData *client_data;
+
+        if (argc != 3) {
+            sprintf(interp->result,
+                "wrong # args: should be \"%.50s modifyVerifyCallback callback\"",
+                argv[0]);
+            return TCL_ERROR;
+        }
+        client_data = (Tm_ClientData *) XtMalloc(sizeof(Tm_ClientData));
+        client_data->callback_func = XtNewString(argv[2]);
+        client_data->widget_info = wPtr;
+
+	XtAddCallback(wPtr->widget, XmNmodifyVerifyCallback,
+			Tm_TextVerifyCallbackHandler, (XtPointer) client_data);
+        XtAddCallback(wPtr->widget, XmNdestroyCallback, 
+                Tm_DestroyReclaimHandler, (XtPointer) client_data);
+
+    } else {
+        return Tm_AnyWidgetCmd(clientData, interp, argc, argv);
+    }
+    return TCL_OK;
+}
+#endif /* XmVERSION >= 2 */
+
 /*
  *--------------------------------------------------------------
  *
@@ -1859,7 +2697,7 @@ Tm_ShellWidgetCmd (clientData, interp, argc, argv)
 	XtPopup(w, XtGrabNone);
 	return TCL_OK;
     } else
-    if (strcmp(argv[1], "popup") == 0) {
+    if (strcmp(argv[1], "popdown") == 0) {
 	XtPopdown(w);
 	return TCL_OK;
     } else {
