@@ -463,8 +463,9 @@ Tm_CvtXmStringToString(display, args, num_args, fromVal, toVal, destructor_data)
     /* whatever buf has grown to, or started from, 
        make it a reasonable size
      */
+    XtFree(buf);
     size = 128;
-    buf = XtRealloc(buf, size);
+    buf = XtMalloc(size);
     p = buf;
 
     xmstr = *(XmString *) fromVal->addr;
@@ -486,8 +487,8 @@ Tm_CvtXmStringToString(display, args, num_args, fromVal, toVal, destructor_data)
 	p += segment_len;
 	if (separator == True) {
 	    *p++ = '\n';
-	    *p = 0;
 	}
+	*p = 0;
 	XtFree(text);
     }
     XmStringFreeContext(context);
@@ -530,12 +531,13 @@ Tm_XmStringToString(xmstr)
     /* whatever buf has grown to, or started from, 
        make it a reasonable size
      */
+    XtFree(buf);
     size = 128;
-    buf = XtRealloc(buf, size);
+    buf = XtMalloc(size);
     p = buf;
 
     if (!XmStringInitContext(&context, xmstr)) {
-	return False;
+	return "";
     }
     while (XmStringGetNextSegment(context, &text, &charset,
 		&direction, &separator)) {
@@ -552,8 +554,8 @@ Tm_XmStringToString(xmstr)
 	p += segment_len;
 	if (separator == True) {
 	    *p++ = '\n';
-	    *p = 0;
 	}
+	*p = 0;
 	XtFree(text);
     }
     XmStringFreeContext(context);
@@ -747,7 +749,8 @@ Boolean            sep;
 	}
     }
 
-static XmString    
+#if XmVERSION < 2
+XmString    
 Tm_StringToXmString(interp, inString)
     char  *inString;
     Tcl_Interp *interp;
@@ -903,6 +906,236 @@ Tm_StringToXmString(interp, inString)
     return result;
 }
 
+#else /* XmVERSION < 2 */
+
+XmString    
+Tm_StringToXmString(interp, inString)
+    char  *inString;
+    Tcl_Interp *interp;
+{
+    char              *segBuf;
+    XmString          result = NULL;
+    XmString	      xmstr;
+    XmStringDirection  dir;
+    char              *segPtr;
+    char              *ptr;
+    int 	      argc;
+    char	      **argv;
+    int		      n;
+    Boolean	      segment_started = True;
+    int		      len;
+
+    len = strlen(inString); 
+    segPtr = segBuf =  XtMalloc(len + 1);
+
+    /* In Motif 2.0 the segment stuff will vanish, leaving
+       the component stuff only
+     */
+    *segPtr = '\0';
+
+    Tcl_SplitList(interp, inString, &argc, &argv);
+    for (n = 0; n < argc; n++) {
+	ptr = argv[n];
+	switch (*ptr) {
+	case '\n':
+	    /* NewLine: Finish current segment */
+
+	    *segPtr = '\0';
+	    xmstr = XmStringComponentCreate(XmSTRING_COMPONENT_TEXT,
+				strlen(segBuf), segBuf);
+	    result = XmStringConcatAndFree(result, xmstr);
+
+	    xmstr = XmStringComponentCreate(XmSTRING_COMPONENT_SEPARATOR,
+				0, NULL);
+	    result = XmStringConcatAndFree(result, xmstr);
+	    segPtr = segBuf;
+	    segment_started = True;
+	    break;
+	    
+	case TM_FORMAT_CHAR:
+            /* Enter command mode */
+	    ptr++;
+
+	    if (*ptr == '\0') {
+		fprintf(stderr, 
+		    "XmString Conversion: Cannot handle trailing 'TM_FORMAT_CHAR'\n");
+		return NULL;
+	    }
+
+	    if (*ptr == TM_FORMAT_CHAR) {
+		/* Two TM_FORMAT_CHAR => leave command mode, 
+		   insert one TM_FORMAT_CHAR */
+		if (*segPtr != '\0')
+		    *segPtr++ = ' ';
+		*segPtr++ = TM_FORMAT_CHAR;
+	    } else
+
+	    switch (*ptr) {
+		case 'n':
+		    *segPtr = '\0';
+		    xmstr = XmStringComponentCreate(XmSTRING_COMPONENT_TEXT,
+					strlen(segBuf), segBuf);
+		    result = XmStringConcatAndFree(result, xmstr);
+
+		    xmstr = XmStringComponentCreate(XmSTRING_COMPONENT_SEPARATOR,
+				0, NULL);
+		    result = XmStringConcatAndFree(result, xmstr);
+		    segPtr = segBuf;
+		    segment_started = True;
+		    break;
+		case 'F':
+		case 'f':
+		    /* save string collected so far */
+		    *segPtr = '\0';
+		    xmstr = XmStringComponentCreate(XmSTRING_COMPONENT_TEXT,
+					strlen(segBuf), segBuf);
+		    result = XmStringConcatAndFree(result, xmstr);
+
+		    /* now make new font */
+		    ptr++;
+		    if (*ptr == '{') {
+			/* @f{font-name} */
+			ptr[strlen(ptr) - 1] = '\0'; /* lose } */
+			ptr++;
+		    } else
+		    if (*ptr == '(') {
+			/* @fXY - 2 char font name XY */
+			ptr++;
+			ptr[2] = '\0'; /* should be anyway */
+		    } else {
+			/* @fX - 1 char font name X */
+			ptr[1] = '\0'; /* should be anyway */
+		    }
+		    xmstr = XmStringComponentCreate(XmSTRING_COMPONENT_TAG,
+					strlen(ptr), ptr);
+		    result = XmStringConcatAndFree(result, xmstr);
+
+		    segPtr = segBuf;
+	    	    segment_started = True;
+		    *segPtr = '\0';
+		    break;
+		case 'B':
+		case 'b':
+		    /* save string collected so far */
+		    *segPtr = '\0';
+		    xmstr = XmStringComponentCreate(XmSTRING_COMPONENT_TEXT,
+					strlen(segBuf), segBuf);
+		    result = XmStringConcatAndFree(result, xmstr);
+
+		    /* now make new font */
+		    ptr++;
+		    if (*ptr == '{') {
+			/* @f{font-name} */
+			ptr[strlen(ptr) - 1] = '\0'; /* lose } */
+			ptr++;
+		    } else
+		    if (*ptr == '(') {
+			/* @fXY - 2 char font name XY */
+			ptr++;
+			ptr[2] = '\0'; /* should be anyway */
+		    } else {
+			/* @fX - 1 char font name X */
+			ptr[1] = '\0'; /* should be anyway */
+		    }
+		    xmstr = XmStringComponentCreate(
+					XmSTRING_COMPONENT_RENDITION_BEGIN,
+					strlen(ptr), ptr);
+		    result = XmStringConcatAndFree(result, xmstr);
+
+		    segPtr = segBuf;
+	    	    segment_started = True;
+		    *segPtr = '\0';
+		    break;
+		case 'E':
+		case 'e':
+		    /* save string collected so far */
+		    *segPtr = '\0';
+		    xmstr = XmStringComponentCreate(XmSTRING_COMPONENT_TEXT,
+					strlen(segBuf), segBuf);
+		    result = XmStringConcatAndFree(result, xmstr);
+
+		    /* now make new font */
+		    ptr++;
+		    if (*ptr == '{') {
+			/* @f{font-name} */
+			ptr[strlen(ptr) - 1] = '\0'; /* lose } */
+			ptr++;
+		    } else
+		    if (*ptr == '(') {
+			/* @fXY - 2 char font name XY */
+			ptr++;
+			ptr[2] = '\0'; /* should be anyway */
+		    } else {
+			/* @fX - 1 char font name X */
+			ptr[1] = '\0'; /* should be anyway */
+		    }
+		    xmstr = XmStringComponentCreate(
+					XmSTRING_COMPONENT_RENDITION_END,
+					strlen(ptr), ptr);
+		    result = XmStringConcatAndFree(result, xmstr);
+
+		    segPtr = segBuf;
+	    	    segment_started = True;
+		    *segPtr = '\0';
+		    break;
+		case 'L':
+		case 'l':
+	    	    *segPtr = '\0';
+		    xmstr = XmStringComponentCreate(XmSTRING_COMPONENT_TEXT,
+					strlen(segBuf), segBuf);
+		    result = XmStringConcatAndFree(result, xmstr);
+
+		    dir = XmSTRING_DIRECTION_L_TO_R;
+		    xmstr = XmStringDirectionCreate(dir);
+		    result = XmStringConcatAndFree(result, xmstr);
+
+		    segPtr = segBuf;
+	    	    segment_started = True;
+		    break;
+		case 'R':
+		case 'r':
+	    	    *segPtr = '\0';
+		    xmstr = XmStringComponentCreate(XmSTRING_COMPONENT_TEXT,
+					strlen(segBuf), segBuf);
+		    result = XmStringConcatAndFree(result, xmstr);
+
+		    dir = XmSTRING_DIRECTION_R_TO_L;
+		    xmstr = XmStringDirectionCreate(dir);
+		    result = XmStringConcatAndFree(result, xmstr);
+
+		    segPtr = segBuf;
+	    	    segment_started = True;
+		    break;
+		default:
+		    break;
+	    }
+	    break;
+	    	    
+	default:
+	    /* insert a space if not first */
+	    if ( ! segment_started)
+		*segPtr++ = ' ';
+	    /* and copy string across */
+	    while (*ptr)
+	        *segPtr++ = *ptr++;
+	    segment_started = False;
+	    break;
+
+	} /* switch */	    
+    } /* for */
+
+    *segPtr = '\0';
+    xmstr = XmStringComponentCreate(XmSTRING_COMPONENT_TEXT,
+				strlen(segBuf), segBuf);
+    result = XmStringConcatAndFree(result, xmstr);
+
+    free((char *) argv);
+    XtFree(segBuf);
+    return result;
+}
+
+#endif /* XmVERSION < 2 */
+
 
 
 static Boolean
@@ -1462,6 +1695,62 @@ Tm_CvtStringToCallbackProc(display, args, num_args, fromVal, toVal, destructor_d
 #endif
 }
 
+/*
+ *--------------------------------------------------------------
+ *
+ * Tm_CvtStringToWidgetClass --
+ *
+ *      Converter from String to WidgetClass
+ *
+ * Results:
+ *
+ *      changes the "toVal" arg to contain the new value.
+ *
+ * Side effects:
+ *
+ *      none
+ *--------------------------------------------------------------
+ */
+
+#if 1
+Boolean
+Tm_CvtStringToWidgetClass(display, args, num_args, fromVal,
+		toVal, destructor_data)
+    Display *display;
+    XrmValuePtr args;
+    Cardinal *num_args;
+    XrmValuePtr fromVal;
+    XrmValuePtr toVal;
+    XtPointer *destructor_data;
+{
+    String str_class = (char *) fromVal->addr;
+    static WidgetClass wc;
+    int n = 0;
+    extern Tm_CommandToClassType Tm_CommToClass[];
+    extern Tm_CommandToClassType Tm_ExternCommandToClass[];
+
+    while (Tm_CommToClass[n].command != NULL) {
+	wc = *(Tm_CommToClass[n].clas);
+	if (strcmp(wc->core_class.class_name, str_class) == 0) {
+	    done(&wc, WidgetClass);
+	    return True;
+	}
+	n++;
+    }
+
+    n = 0;
+    while (Tm_ExternCommandToClass[n].command != NULL) {
+	wc = *(Tm_ExternCommandToClass[n].clas);
+	if (strcmp(wc->core_class.class_name, str_class) == 0) {
+	    done(&wc, WidgetClass);
+	    return True;
+	}
+	n++;
+    }
+
+    return False;
+}
+#endif
 
 /*
  *--------------------------------------------------------------
@@ -1949,4 +2238,14 @@ Tm_RegisterConverters(interp, appContext)
 		2,
 		XtCacheNone,
 		NULL);
+#if 1
+
+    /*
+     * String to WidgetClass
+     */
+    XtSetTypeConverter(
+		XmRString, XmRWidgetClass,
+		Tm_CvtStringToWidgetClass,
+		NULL, 0, XtCacheNone, NULL);
+#endif
 }
