@@ -62,6 +62,56 @@ static char rcsid[] = "$Header: /usrs/tm/RCS/tmPushButton.c,v 1.2 1993/07/14 20:
 
 #include <X11/Shell.h>
 
+/*
+ * this list allows one to move from the widget creation string
+ * to the Xt class so that it can be initialised correctly
+ */
+Tm_CommandToClassType Tm_CommToClass[] = {
+    {xmArrowButton, &xmArrowButtonWidgetClass},
+    {xmBulletinBoard, &xmBulletinBoardWidgetClass, },
+    {xmCascadeButton, &xmCascadeButtonWidgetClass},
+    {xmCommand, &xmCommandWidgetClass},
+    {xmDialogShell, &xmDialogShellWidgetClass},
+    {xmDrawingArea, &xmDrawingAreaWidgetClass},
+    {xmDrawnButton, &xmDrawnButtonWidgetClass},
+    {xmFileSelectionBox, &xmFileSelectionBoxWidgetClass},
+    {xmForm, &xmFormWidgetClass},
+    {xmFrame, &xmFrameWidgetClass},
+    {xmList, &xmListWidgetClass},
+    {xmLabel, &xmLabelWidgetClass},
+    {xmList, &xmListWidgetClass},
+    {xmMainWindow, &xmMainWindowWidgetClass},
+    {xmMessageBox, &xmMessageBoxWidgetClass},
+    {xmPanedWindow, &xmPanedWindowWidgetClass},
+    {xmPushButton, &xmPushButtonWidgetClass},
+    {xmRowColumn, &xmRowColumnWidgetClass},
+    {xmScale, &xmScaleWidgetClass},
+    {xmScrollBar, &xmScrollBarWidgetClass},
+    {xmScrolledWindow, &xmScrolledWindowWidgetClass},
+    {xmSelectionBox, &xmSelectionBoxWidgetClass},
+    {xmSeparator, &xmSeparatorWidgetClass},
+    {xmText, &xmTextWidgetClass},
+    {xmTextField, &xmTextFieldWidgetClass},
+    {xmToggleButton, &xmToggleButtonWidgetClass},
+    {xmTopLevelShell, &topLevelShellWidgetClass},
+
+#if XmVERSION >= 2
+    {xmComboBox, &xmComboBoxWidgetClass},
+    {xmContainer, &xmContainerWidgetClass},
+    {xmCSText, &xmCSTextWidgetClass},
+    {xmIconGadget, &xmIconGadgetClass},
+    {xmNotebook, &xmNotebookWidgetClass},
+    {xmSpinBox, &xmSpinBoxWidgetClass},
+#endif /* XmVERSION */
+
+    {NULL, NULL}
+};
+
+/* this list will be supplied by someone else
+ * defaults to no extra widgets
+ */
+extern Tm_CommandToClassType Tm_ExternCommandToClass[];
+
 /*
  * this is the list of resources that require the widget rather
  * than its parent for proper conversions from String.
@@ -71,7 +121,7 @@ static char rcsid[] = "$Header: /usrs/tm/RCS/tmPushButton.c,v 1.2 1993/07/14 20:
  * code, how am I supposed to know what is in this list?
  * Is it dependant on Motif versions?
  *
- * OSF should leap into action and supply a fix for this
+ * OSF should leap into action and supply a fix for this :-)
  */
 static char *dont_use_parent_for_resources[] = {
 	"width",
@@ -221,10 +271,11 @@ Tm_AnyCmd(clientData, interp, argc, argv)
     int num_args;
     char *parent_argv[TM_MAXARGS], *child_argv[TM_MAXARGS];
     int num_parent_argc, num_child_argc;
-    WidgetClass class;
+    WidgetClass class = NULL;
     Tm_WidgetCmdProc widgetCmdProc;
     Boolean managed = False;
     Tcl_CmdInfo parentInfo;
+    int n;
  
     widgetCmdProc = (Tm_WidgetCmdProc) clientData;
 
@@ -232,6 +283,7 @@ Tm_AnyCmd(clientData, interp, argc, argv)
     if (parent == NULL)
 	return TCL_ERROR;
 
+#if 0
     if (strcmp(command, xmArrowButton) == 0) {
 	class = xmArrowButtonWidgetClass;
     } else
@@ -306,9 +358,55 @@ Tm_AnyCmd(clientData, interp, argc, argv)
     } else
     if (strcmp(command, xmTopLevelShell) == 0) {
 	class = topLevelShellWidgetClass;
+
+#if XmVERSION >= 2
+    } else
+    if (strcmp(command, xmContainer) == 0) {
+	class = xmContainerWidgetClass;
+    } else
+    if (strcmp(command, xmCSText) == 0) {
+	class = xmCSTextWidgetClass;
+    } else
+    if (strcmp(command, xmIconGadget) == 0) {
+	class = xmIconGadgetClass;
+    } else
+    if (strcmp(command, xmNotebook) == 0) {
+	class = xmNotebookWidgetClass;
+    } else
+    if (strcmp(command, xmSpinBox) == 0) {
+	class = xmSpinBoxWidgetClass;
+#endif /* XmVERSION */
+
     } else {
 	return TCL_ERROR;
     }
+#endif /* 0 */
+
+    /* check if it is one of the tclMotif commands */
+    n = 0;
+    while (Tm_CommToClass[n].command != NULL) {
+	if (strcmp(Tm_CommToClass[n].command, command) == 0) {
+	    class = *(Tm_CommToClass[n].clas);
+	    break;
+	}
+	n++;
+    }
+
+    /* check if it is one of the external widgets */
+    if (class == NULL) {
+        n = 0;
+        while (Tm_ExternCommandToClass[n].command != NULL) {
+	    if (strcmp(Tm_ExternCommandToClass[n].command, command) == 0) {
+	        class = *(Tm_ExternCommandToClass[n].clas);
+	        break;
+	    }
+	    n++;
+	}
+    }
+    if (class == NULL) {
+	sprintf(interp->result, "unknown class %s", command);
+	return TCL_ERROR;
+    }
 
     if (argc >= 3 && strcmp(argv[2], "managed") == 0) {
 	managed = True;
@@ -325,11 +423,13 @@ Tm_AnyCmd(clientData, interp, argc, argv)
     /* set up for garbage collect of some resource values */
     Tm_InitFreeResourceList((argc - 2)/2);
 
-    Tm_SetValues(path, interp, parent, parent, class,
-				parent_argv, num_parent_argc, args, &num_args);
+    if (Tm_SetValues(path, interp, parent, parent, class,
+		parent_argv, num_parent_argc, args, &num_args) == TCL_ERROR)
+	return TCL_ERROR;
     new = XtCreateWidget (Tm_NameFromPath(path), class, parent, args, num_args);
-    Tm_SetValues(path, interp, new, parent, class,
-				child_argv, num_child_argc, args, &num_args);
+    if (Tm_SetValues(path, interp, new, parent, class,
+		child_argv, num_child_argc, args, &num_args) == TCL_ERROR)
+	return TCL_ERROR;
     XtSetValues(new, args, num_args);
 
     /* collect garbage */
@@ -389,6 +489,11 @@ CreateChild(interp, path, w, name)
     char *subobject_path;
     Tm_Widget *wPtr;
     Tcl_CmdInfo cmdInfo;
+    Tcl_CmdProc *widgetCmdProc = NULL;
+    WidgetClass class;
+    String widgetCmd = NULL;
+    int n;
+    Tm_Cmd *cmdPtr;
 
     subobject_path = XtMalloc(strlen(path) + strlen(name) + 2);
     strcpy(subobject_path, path);
@@ -408,7 +513,61 @@ CreateChild(interp, path, w, name)
     Tm_StoreWidgetInfo(subobject_path, wPtr, interp);
     XtVaSetValues(w, XmNuserData, wPtr, NULL);
 
-    Tcl_CreateCommand (interp, subobject_path, Tm_AnyWidgetCmd,
+    /* 
+     * the widget that has been created by Motif as a child widget
+     * may be a gadget, an "ordinary" widget, or one that has special
+     * methods like a Text or List widget (eg List child of SelectionBox.
+     * So we should set the appropriate method handler for this.
+     * This means using the class pointer to find the command name,
+     * and then the command name to find the method handler. Bit slow...
+     */
+    class = XtClass(w);
+    n = 0;
+    /* try Motif list */
+    while (Tm_CommToClass[n].command != NULL) {
+	if (class == *(Tm_CommToClass[n].clas)) {
+	    widgetCmd = Tm_CommToClass[n].command;
+	    break;
+	}
+	n++;
+    }
+    if (widgetCmd == NULL) {
+	/* try extension widgets list */
+	n  = 0;
+        while (Tm_ExternCommandToClass[n].command != NULL) {
+	    if (class == *(Tm_ExternCommandToClass[n].clas)) {
+	        widgetCmd = Tm_ExternCommandToClass[n].command;
+	        break;
+	    }
+	    n++;
+	}
+    }
+    if (widgetCmd != NULL) {
+	/* It is one of the known widgets, so try to find the 
+	 * method handler for it
+	 */
+        for (cmdPtr = Tm_Commands; cmdPtr->name != NULL; cmdPtr++) {
+	    if (strcmp(cmdPtr->name, widgetCmd) == 0) {
+		widgetCmdProc = cmdPtr->widgetCmdProc;
+		break;
+	    }
+	}
+	if (widgetCmdProc != NULL) {
+            for (cmdPtr = Tm_ExternCommands; cmdPtr->name != NULL; cmdPtr++) {
+                if (strcmp(cmdPtr->name, widgetCmd) == 0) {
+                    widgetCmdProc = cmdPtr->widgetCmdProc;
+                    break;
+                }
+	    }
+	}
+    }
+
+    /* if we haven't found it now, then we treat it as a general widget */
+    if (widgetCmdProc == NULL) {
+	widgetCmdProc = Tm_AnyWidgetCmd;
+    }
+
+    Tcl_CreateCommand (interp, subobject_path, widgetCmdProc,
     	 (ClientData) wPtr, (void (*) ()) NULL);
         
     XtAddCallback(w, XmNdestroyCallback, Tm_DestroyWidgetHandler,
@@ -701,96 +860,108 @@ Tm_DialogCmd(clientData, interp, argc, argv)
     if (strcmp(command, xmBulletinBoardDialog) == 0) {
 	class = xmBulletinBoardWidgetClass;
         XtInitializeWidgetClass(class);
-        Tm_SetValues(path, interp, parent, parent, class,
-				parent_argv, num_parent_argc, args, &num_args);
+        if (Tm_SetValues(path, interp, parent, parent, class,
+		parent_argv, num_parent_argc, args, &num_args) == TCL_ERROR)
+	    return TCL_ERROR;
 	new = XmCreateBulletinBoardDialog (parent,
 			Tm_NameFromPath(path), args, num_args);
     } else
     if (strcmp(command, xmFileSelectionDialog) == 0) {
 	class = xmFileSelectionBoxWidgetClass;
         XtInitializeWidgetClass(class);
-        Tm_SetValues(path, interp, parent, parent, class,
-				parent_argv, num_parent_argc, args, &num_args);
+        if (Tm_SetValues(path, interp, parent, parent, class,
+		parent_argv, num_parent_argc, args, &num_args) == TCL_ERROR)
+	    return TCL_ERROR;
 	new = XmCreateFileSelectionDialog (parent,
 			Tm_NameFromPath(path), args, num_args);
     } else
     if (strcmp(command, xmFormDialog) == 0) {
 	class = xmFormWidgetClass;
         XtInitializeWidgetClass(class);
-        Tm_SetValues(path, interp, parent, parent, class,
-				parent_argv, num_parent_argc, args, &num_args);
+        if (Tm_SetValues(path, interp, parent, parent, class,
+		parent_argv, num_parent_argc, args, &num_args) == TCL_ERROR)
+	    return TCL_ERROR;
 	new = XmCreateFormDialog (parent,
 			Tm_NameFromPath(path), args, num_args);
     } else
     if (strcmp(command, xmErrorDialog) == 0) {
 	class = xmMessageBoxWidgetClass;
         XtInitializeWidgetClass(class);
-        Tm_SetValues(path, interp, parent, parent, class,
-				parent_argv, num_parent_argc, args, &num_args);
+        if (Tm_SetValues(path, interp, parent, parent, class,
+		parent_argv, num_parent_argc, args, &num_args) == TCL_ERROR)
+	    return TCL_ERROR;
 	new = XmCreateErrorDialog (parent, 
 			Tm_NameFromPath(path), args, num_args);
     } else 
     if (strcmp(command, xmInformationDialog) == 0) {
 	class = xmMessageBoxWidgetClass;
         XtInitializeWidgetClass(class);
-        Tm_SetValues(path, interp, parent, parent, class,
-				parent_argv, num_parent_argc, args, &num_args);
+        if (Tm_SetValues(path, interp, parent, parent, class,
+		parent_argv, num_parent_argc, args, &num_args) == TCL_ERROR)
+	    return TCL_ERROR;
 	new = XmCreateInformationDialog (parent, 
 			Tm_NameFromPath(path), args, num_args);
     } else
     if (strcmp(command, xmMessageDialog) == 0) {
 	class = xmMessageBoxWidgetClass;
         XtInitializeWidgetClass(class);
-        Tm_SetValues(path, interp, parent, parent, class,
-				parent_argv, num_parent_argc, args, &num_args);
+        if (Tm_SetValues(path, interp, parent, parent, class,
+		parent_argv, num_parent_argc, args, &num_args) == TCL_ERROR)
+	    return TCL_ERROR;
 	new = XmCreateMessageDialog (parent, 
 			Tm_NameFromPath(path), args, num_args);
     } else
     if (strcmp(command, xmQuestionDialog) == 0) {
 	class = xmMessageBoxWidgetClass;
         XtInitializeWidgetClass(class);
-        Tm_SetValues(path, interp, parent, parent, class,
-				parent_argv, num_parent_argc, args, &num_args);
+        if (Tm_SetValues(path, interp, parent, parent, class,
+		parent_argv, num_parent_argc, args, &num_args) == TCL_ERROR)
+	    return TCL_ERROR;
 	new = XmCreateQuestionDialog (parent, 
 			Tm_NameFromPath(path), args, num_args);
     } else
     if (strcmp(command, xmWarningDialog) == 0) {
 	class = xmMessageBoxWidgetClass;
         XtInitializeWidgetClass(class);
-        Tm_SetValues(path, interp, parent, parent, class,
-				parent_argv, num_parent_argc, args, &num_args);
+        if (Tm_SetValues(path, interp, parent, parent, class,
+		parent_argv, num_parent_argc, args, &num_args) == TCL_ERROR)
+	    return TCL_ERROR;
 	new = XmCreateWarningDialog (parent, 
 			Tm_NameFromPath(path), args, num_args);
     } else
     if (strcmp(command, xmWorkingDialog) == 0) {
 	class = xmMessageBoxWidgetClass;
         XtInitializeWidgetClass(class);
-        Tm_SetValues(path, interp, parent, parent, class,
-				parent_argv, num_parent_argc, args, &num_args);
+        if (Tm_SetValues(path, interp, parent, parent, class,
+		parent_argv, num_parent_argc, args, &num_args) == TCL_ERROR)
+	    return TCL_ERROR;
 	new = XmCreateWorkingDialog (parent, 
 			Tm_NameFromPath(path), args, num_args);
     } else
     if (strcmp(command, xmPromptDialog) == 0) {
 	class = xmSelectionBoxWidgetClass;
         XtInitializeWidgetClass(class);
-        Tm_SetValues(path, interp, parent, parent, class,
-				parent_argv, num_parent_argc, args, &num_args);
+        if (Tm_SetValues(path, interp, parent, parent, class,
+		parent_argv, num_parent_argc, args, &num_args) == TCL_ERROR)
+	    return TCL_ERROR;
 	new = XmCreatePromptDialog (parent, 
 			Tm_NameFromPath(path), args, num_args);
     } else
     if (strcmp(command, xmSelectionDialog) == 0) {
 	class = xmSelectionBoxWidgetClass;
         XtInitializeWidgetClass(class);
-        Tm_SetValues(path, interp, parent, parent, class,
-				parent_argv, num_parent_argc, args, &num_args);
+        if (Tm_SetValues(path, interp, parent, parent, class,
+		parent_argv, num_parent_argc, args, &num_args) == TCL_ERROR)
+	    return TCL_ERROR;
 	new = XmCreateSelectionDialog (parent, 
 			Tm_NameFromPath(path), args, num_args);
     } else {
 	return TCL_ERROR;
     }
 
-    Tm_SetValues(path, interp, new, parent, class,
-                                child_argv, num_child_argc, args, &num_args);
+    if (Tm_SetValues(path, interp, new, parent, class,
+                child_argv, num_child_argc, args, &num_args) == TCL_ERROR)
+	return TCL_ERROR;
     XtSetValues(new, args, num_args);
 
 
@@ -957,23 +1128,26 @@ Tm_ScrolledCmd(clientData, interp, argc, argv)
     if (strcmp(command, xmScrolledList) == 0) {
 	class = xmListWidgetClass;
         XtInitializeWidgetClass(class);
-        Tm_SetValues(path, interp, parent, parent, class,
-				parent_argv, num_parent_argc, args, &num_args);
+        if (Tm_SetValues(path, interp, parent, parent, class,
+		parent_argv, num_parent_argc, args, &num_args) == TCL_ERROR)
+	    return TCL_ERROR;
         new = XmCreateScrolledList (parent,
 			Tm_NameFromPath(path), args, num_args);
     } else
     if (strcmp(command, xmScrolledText) == 0) {
 	class = xmTextWidgetClass;
         XtInitializeWidgetClass(class);
-        Tm_SetValues(path, interp, parent, parent, class,
-				parent_argv, num_parent_argc, args, &num_args);
+        if (Tm_SetValues(path, interp, parent, parent, class,
+		parent_argv, num_parent_argc, args, &num_args) == TCL_ERROR)
+	    return TCL_ERROR;
         new = XmCreateScrolledText (parent, Tm_NameFromPath(path), args, num_args);
     } else {
 	return TCL_ERROR;
     }
 
-    Tm_SetValues(path, interp, new, parent, class,
-                                child_argv, num_child_argc, args, &num_args);
+    if (Tm_SetValues(path, interp, new, parent, class,
+                child_argv, num_child_argc, args, &num_args) == TCL_ERROR)
+	return TCL_ERROR;
     XtSetValues(new, args, num_args);
 
     if (managed) {
@@ -1064,6 +1238,7 @@ Tm_RowColumnCmd(clientData, interp, argc, argv)
     Boolean managed = False;
     Tcl_CmdInfo parentInfo;
     Tm_WidgetCmdProc widgetCmdProc;
+    Widget tearoff;
 
     widgetCmdProc = (Tm_WidgetCmdProc) clientData;
 
@@ -1083,8 +1258,9 @@ Tm_RowColumnCmd(clientData, interp, argc, argv)
     DivideArgs(argv + 2, argc - 2, parent_argv, &num_parent_argc,
                         child_argv, &num_child_argc);
 
-    Tm_SetValues(path, interp, parent, parent, xmRowColumnWidgetClass,
-				parent_argv, num_parent_argc, args, &num_args);
+    if (Tm_SetValues(path, interp, parent, parent, xmRowColumnWidgetClass,
+		parent_argv, num_parent_argc, args, &num_args) == TCL_ERROR)
+	return TCL_ERROR;
     if (strcmp(command, xmRowColumn) == 0) {
         new = XmCreateRowColumn (parent,
 			Tm_NameFromPath(path), args, num_args);
@@ -1109,8 +1285,128 @@ Tm_RowColumnCmd(clientData, interp, argc, argv)
 			command);
 	return TCL_ERROR;
     }
-    Tm_SetValues(path, interp, new, parent, xmRowColumnWidgetClass,
-                                child_argv, num_child_argc, args, &num_args);
+    if (Tm_SetValues(path, interp, new, parent, xmRowColumnWidgetClass,
+               child_argv, num_child_argc, args, &num_args) == TCL_ERROR)
+	return TCL_ERROR;
+    XtSetValues(new, args, num_args);
+
+    if (managed) {
+        XtManageChild (new);
+    }
+
+    wPtr = (Tm_Widget *) XtMalloc (sizeof (Tm_Widget));
+    wPtr -> interp = interp;
+    wPtr -> widget = new;
+    wPtr -> pathName = XtNewString(path);
+    wPtr -> parent = Tm_ParentPath(path);
+    wPtr -> dropProc = NULL;
+
+    Tcl_GetCommandInfo(interp, wPtr->parent, &parentInfo);
+    wPtr -> displayInfo = ((Tm_Widget *) parentInfo.clientData)->displayInfo;
+
+    Tm_StoreWidgetInfo(path, wPtr, interp);
+    XtVaSetValues(new, XmNuserData, wPtr, NULL);
+
+    Tcl_CreateCommand (interp, path, widgetCmdProc,
+		 (ClientData) wPtr, (void (*) ()) NULL);
+
+    XtAddCallback(new, XmNdestroyCallback, Tm_DestroyWidgetHandler,
+	(XtPointer) wPtr);
+
+    interp -> result = wPtr -> pathName;
+
+    /* if we are a pulldown or popup, we may be a tearoff menu.
+       In that case a tearoff button may have been created for us,
+       and we need to add this child to tm
+     */
+    if ((tearoff = XmGetTearOffControl(new)) != NULL) {
+	CreateChild(interp, path, tearoff, "TearOffControl");
+    }
+
+    return TCL_OK;
+}
+
+#if XmVERSION >= 2
+
+/*
+ *--------------------------------------------------------------
+ *
+ * Tm_ComboBoxCmd --
+ *
+ *	This procedure is invoked to process the "comboBox",
+ *	convenience Tcl commands.  See the
+ *	user documentation for details on what it does.
+ *
+ * Results:
+ *	A standard Tcl result.
+ *
+ * Side effects:
+ *	See the user documentation.
+ *
+ *--------------------------------------------------------------
+ */
+
+/* ARGSUSED */
+int
+Tm_ComboBoxCmd(clientData, interp, argc, argv)
+    ClientData clientData;	/* Main window associated with
+				 * interpreter. */
+    Tcl_Interp *interp;		/* Current interpreter. */
+    int argc;			/* Number of arguments. */
+    char **argv;		/* Argument strings. */
+{
+    char *command = argv[0];
+    register Tm_Widget *wPtr;
+    Widget new, parent;
+    char *path = argv[1];
+    Arg args[TM_MAXARGS];
+    int num_args;
+    char *parent_argv[TM_MAXARGS], *child_argv[TM_MAXARGS];
+    int num_parent_argc, num_child_argc;
+    Boolean managed = False;
+    Tcl_CmdInfo parentInfo;
+    Tm_WidgetCmdProc widgetCmdProc;
+
+    widgetCmdProc = (Tm_WidgetCmdProc) clientData;
+
+    parent = Tm_ParentWidgetFromPath (interp, path);
+    if (parent == NULL)
+	return TCL_ERROR;
+
+    if (argc >= 3 && strcmp(argv[2], "managed") == 0) {
+        managed = True;
+        argv++;
+        argc--;
+    }
+
+    /* make sure class is initialized before using XtConvertAndStore */
+    XtInitializeWidgetClass(xmComboBoxWidgetClass);
+
+    DivideArgs(argv + 2, argc - 2, parent_argv, &num_parent_argc,
+                        child_argv, &num_child_argc);
+
+    if (Tm_SetValues(path, interp, parent, parent, xmComboBoxWidgetClass,
+		parent_argv, num_parent_argc, args, &num_args) == TCL_ERROR)
+	return TCL_ERROR;
+    if (strcmp(command, xmComboBox) == 0) {
+        new = XmCreateComboBox (parent,
+			Tm_NameFromPath(path), args, num_args);
+    } else 
+    if (strcmp(command, xmDropDownComboBox) == 0) {
+	new = XmCreateDropDownComboBox(parent, 
+			Tm_NameFromPath(path), args, num_args);
+    } else 
+    if (strcmp(command, xmDropDownList) == 0) {
+	new = XmCreateDropDownList(parent, 
+			Tm_NameFromPath(path), args, num_args);
+    } else {
+	sprintf(interp->result, "Unknown xmRowColumn creation command %s\n",
+			command);
+	return TCL_ERROR;
+    }
+    if (Tm_SetValues(path, interp, new, parent, xmComboBoxWidgetClass,
+               child_argv, num_child_argc, args, &num_args) == TCL_ERROR)
+	return TCL_ERROR;
     XtSetValues(new, args, num_args);
 
     if (managed) {
@@ -1139,3 +1435,5 @@ Tm_RowColumnCmd(clientData, interp, argc, argv)
     interp -> result = wPtr -> pathName;
     return TCL_OK;
 }
+
+#endif /* XmVERSION */
