@@ -1,4 +1,6 @@
 /*
+ *      "$Revision: 1.4 $"
+ *
  * TmDrag.c --
  *	This module contains the main set of functions
  *	for drag and drop
@@ -37,7 +39,14 @@ Boolean Tm_SettingDropProc = False;
 Tm_Widget *Tm_HackWidgetPtr; /* see gripe in next function */
 Tm_Widget *Tm_HackDragWidgetPtr;
 
-		
+typedef struct {
+    Atom type;
+    char *(*converter)();
+} DropConverter;
+
+static DropConverter *dropConverters = NULL;
+static int numdc;
+
 /*--------------------------------------------------------------
  *
  * Tm_DragContextCmd --
@@ -94,7 +103,8 @@ Tm_DragContextCmd(clientData, interp, argc, argv)
 	/* hmmm, looks like we need to set some resources *right now*.
 	   Pity the doco doesn't tell me this...
 	 */
-	Tm_SetValues(butPtr->pathName, interp, butPtr->widget, butPtr->widget,
+	Tm_SetValues(butPtr->pathName, interp, butPtr->widget,
+butPtr->widget,
 			class, argv+2, argc-2, args, &num_args);
 	dropTransfer = XmDropTransferStart(butPtr->widget, args, num_args);
 
@@ -107,6 +117,97 @@ Tm_DragContextCmd(clientData, interp, argc, argv)
     return TCL_OK;
 }
 
+/*--------------------------------------------------------------
+ *
+ */
+static char*
+ConvertStringValue(interp, type, value, length)
+Tcl_Interp* interp;
+Atom type;
+XtPointer value;
+int length;
+{
+    char* msg;
+    /* are these null terminated? just in case... */
+    msg = XtMalloc(length + 1);
+    strncpy(msg, value, length);
+    msg[length] = '\0';
+
+    return msg;
+}
+
+static char*
+ConvertCompoundValue(interp, type, value, length)
+Tcl_Interp* interp;
+Atom type;
+XtPointer value;
+int length;
+{
+    char* msg;
+    XmString xmstr;
+    /* I know nothing about Compound Strings, so I'll cheat
+       by letting Motif do the work
+     */
+
+    xmstr = XmCvtCTToXmString(value);
+    XmStringGetLtoR(xmstr, XmFONTLIST_DEFAULT_TAG, &msg);
+    XmStringFree(xmstr);
+
+    return msg;
+}
+
+static void
+InitDropConverters(interp)
+Tcl_Interp* interp;
+{
+    dropConverters = malloc(sizeof(DropConverter)*8);
+    Tm_InstallDropType(interp, "TEXT", ConvertStringValue);
+    Tm_InstallDropType(interp, "STRING", ConvertStringValue);
+    Tm_InstallDropType(interp, "COMPOUND_TEXT", ConvertCompoundValue);
+}
+
+static char*
+ConvertDropData(interp, type, value, length)
+Tcl_Interp *interp;
+Atom type;
+XtPointer value;
+int length;
+{
+    int dcx;
+
+    if (dropConverters == NULL) InitDropConverters(interp);
+
+    for (dcx=0; dcx < numdc; dcx++) {
+	if (dropConverters[dcx].type == type) {
+	    return dropConverters[dcx].converter(interp, type, value,
+length);
+	}
+    }
+    return NULL;
+}
+
+int
+Tm_InstallDropType(interp, typeStr, converter)
+Tcl_Interp *interp;
+char *typeStr;
+char *(*converter)();
+{
+    DropConverter *dc;
+    Tm_Widget* tw = Tm_WidgetInfoFromPath(interp, ".");
+    Display *display = XtDisplay(tw->widget);
+
+    if (dropConverters == NULL) InitDropConverters(interp);
+
+    dropConverters = realloc(dropConverters,
+sizeof(DropConverter)*(numdc+1));
+    dc = &dropConverters[numdc++];
+
+    dc->type = XmInternAtom(display, typeStr, False);
+    dc->converter = converter? converter:ConvertStringValue;
+
+    return TCL_OK;
+}
+
 /*
  *--------------------------------------------------------------
  *
@@ -134,7 +235,8 @@ Tm_DropTransferHandler(w, closure, seltype, type, value, length, format)
 {
     Tcl_Interp *interp;
     Widget droppedOnWidget = (Widget) closure;
-    Tm_Widget *wPtr = Tm_HackWidgetPtr;  /* this is the drag context widget */
+    Tm_Widget *wPtr = Tm_HackWidgetPtr;  /* this is the drag context widget
+*/
     Tm_TransferStruct call_data;
     Atom COMPOUND_TEXT;
     Atom TEXT;
@@ -143,38 +245,13 @@ Tm_DropTransferHandler(w, closure, seltype, type, value, length, format)
     char *new_command;
     char *msg;
 
-
-    COMPOUND_TEXT = XmInternAtom(XtDisplayOfObject(w),
-				"COMPOUND_TEXT", False);
-    TEXT = XmInternAtom(XtDisplayOfObject(w),
-				"TEXT", False);
-    STRING = XmInternAtom(XtDisplayOfObject(w),
-				"STRING", False);
-
     command = wPtr->transferProc;
     interp = wPtr->interp;
 
-    msg = NULL;
-    if (*type == COMPOUND_TEXT) {
-	/* I know nothing about Compound Strings, so I'll cheat
-	   by letting Motif do the work
-	 */
-	XmString xmstr;
-
-	xmstr = XmCvtCTToXmString(value);
-	XmStringGetLtoR(xmstr, XmFONTLIST_DEFAULT_TAG, &msg);
-	XmStringFree(xmstr);
-    } else
-    if (*type == TEXT || *type == STRING) {
-	/* are these null terminated? just in case... */
-	msg = XtMalloc(*length + 1);
-	strncpy(msg, value, *length);
-	msg[*length] = '\0';
-    }
-    call_data.value = msg;
+    call_data.value = ConvertDropData(interp, *type, value, *length);
     call_data.closure = (char *) closure;
 
-    new_command = Tm_ExpandPercents(wPtr->pathName, w, NULL, 
+    new_command = Tm_ExpandPercents(wPtr->pathName, w, NULL,
 				&call_data, command);
 
     if (Tcl_GlobalEval(interp, new_command) != TCL_OK) {
@@ -241,7 +318,8 @@ Tm_DropProcHandler(w, client_data, call_data)
                  (ClientData) dcPtr, (void (*) ()) NULL);
 
     /* this damn thing has no class pointer!!! */
-    command = Tm_ExpandPercents(dcPtr->pathName, dcPtr->widget, dropData->event,
+    command = Tm_ExpandPercents(dcPtr->pathName, dcPtr->widget,
+dropData->event,
 			call_data, dropProc);
 
     if (Tcl_GlobalEval(interp, command) != TCL_OK) {
@@ -293,7 +371,7 @@ Tm_DropSiteSetValues(pathName, interp, w, argv, argc, args, num_args)
     char *resource;
     char *value;
     Tm_ClientData *client_data;
- 
+
     if (argc > TM_MAXARGS*2) {
 	return;
     }
@@ -409,7 +487,8 @@ Tm_DropSiteGetValues(pathName, interp, w, class, argv, argc)
     }
 }
 
-Boolean Tm_ConvertProcHandler(w, selection, target, type, value, length, format)
+Boolean Tm_ConvertProcHandler(w, selection, target, type, value, length,
+format)
     Widget w;
     Atom *selection;
     Atom *target;
