@@ -10,15 +10,20 @@ extern "C" {
 
 /* Widget creation functions */
 extern int Tm_AnyCmd _ANSI_ARGS_((ClientData clientData, Tcl_Interp *interp,
-            int argc, char **argv));
+            int argc, char *argv[]));
 extern int Tm_ScrolledCmd _ANSI_ARGS_((ClientData clientData, Tcl_Interp *interp,
-            int argc, char **argv));
+            int argc, char *argv[]));
 extern int Tm_HasChildrenCmd _ANSI_ARGS_((ClientData clientData, Tcl_Interp *interp,
-            int argc, char **argv));
+            int argc, char *argv[]));
 extern int Tm_DialogCmd _ANSI_ARGS_((ClientData clientData, Tcl_Interp *interp,
-            int argc, char **argv));
+            int argc, char *argv[]));
 extern int Tm_RowColumnCmd _ANSI_ARGS_((ClientData clientData, Tcl_Interp *interp,
+            int argc, char *argv[]));
+
+#if XmVERSION >= 2
+extern int Tm_ComboBoxCmd _ANSI_ARGS_((ClientData clientData, Tcl_Interp *interp,
             int argc, char **argv));
+#endif
 
 /* wdiget specific commands */
 extern int Tm_AnyWidgetCmd _ANSI_ARGS_((ClientData clientData, Tcl_Interp *interp,
@@ -38,12 +43,23 @@ extern int Tm_RootCmd _ANSI_ARGS_((ClientData clientData, Tcl_Interp *interp,
 extern int Tm_ShellWidgetCmd _ANSI_ARGS_((ClientData clientData, Tcl_Interp *interp,
 	     int argc, char **argv));
 
+#if XmVERSION >= 2
+extern int Tm_CSTextWidgetCmd _ANSI_ARGS_((ClientData clientData, Tcl_Interp *interp,
+	     int argc, char **argv));
+extern int Tm_NotebookWidgetCmd _ANSI_ARGS_((ClientData clientData, Tcl_Interp *interp,
+	     int argc, char **argv));
+#endif
+
 /* send commands */
 extern int Tm_SendCmd _ANSI_ARGS_((ClientData clientData, Tcl_Interp *interp,
             int argc, char **argv));
 extern int Tm_RegisterInterp _ANSI_ARGS_((Tcl_Interp *interp, char *name,
 	    Tm_Display *dispPtr));
 
+/* other commands */
+extern int Tm_XEvent _ANSI_ARGS_((ClientData clientData, Tcl_Interp *interp,
+            int argc, char **argv));
+
 /* drag and drop functions */
 extern void Tm_DropSiteSetValues _ANSI_ARGS_((char *path, Tcl_Interp *interp,
              Widget w, char **argv, int argc, Arg args[], int *num_args));
@@ -55,6 +71,8 @@ extern void Tm_DropTransferHandler _ANSI_ARGS_ ((Widget w, XtPointer closure,
 extern Boolean Tm_ConvertProcHandler _ANSI_ARGS_ ((Widget w, Atom *selection,
 	    Atom *target, Atom *type, XtPointer *value, unsigned long *length,
 	     int *format));
+extern int Tm_InstallDropType _ANSI_ARGS_ ((Tcl_Interp *interp,
+	    char *typeStr, char *(*converter) ()));
 	    
 /* result capturing */
 extern int Tm_SaveResult _ANSI_ARGS_ ((Tcl_Interp *interp));
@@ -73,6 +91,18 @@ extern void Tm_DestroyReclaimHandler _ANSI_ARGS_((Widget w,
 	     XtPointer client_data, XtPointer call_data));
 extern void Tm_TextVerifyCallbackHandler _ANSI_ARGS_((Widget w, 
 	     XtPointer client_data, XtPointer call_data));
+#if XmVERSION >= 2
+extern void Tm_CSTextVerifyCallbackHandler _ANSI_ARGS_((Widget w, 
+	     XtPointer client_data, XtPointer call_data));
+extern void Tm_DestinationCallbackHandler _ANSI_ARGS_((Widget w, 
+	     XtPointer client_data, XtPointer call_data));
+extern void Tm_ConvertCallbackHandler _ANSI_ARGS_((Widget w, 
+	     XtPointer client_data, XtPointer call_data));
+#endif
+#if USE_UIL
+extern void Tm_UILCallbackHandler _ANSI_ARGS_((Widget w, 
+	     XtPointer client_data, XtPointer call_data));
+#endif
 
 /* resource free functions */
 extern void Tm_FreeResourceValues _ANSI_ARGS_((void));
@@ -118,6 +148,14 @@ extern int Tm_ResourceList _ANSI_ARGS_ ((Tcl_Interp *interp, Widget w,
 extern int Tm_GetAppResources _ANSI_ARGS_ ((Tcl_Interp *interp, Widget w,
 	     String resource_list));
 extern int Tm_Init _ANSI_ARGS_ ((Tcl_Interp *interp));
+extern String Tm_XmStringToString _ANSI_ARGS_ ((XmString xmstr));
+extern XmString Tm_StringToXmString _ANSI_ARGS_ ((Tcl_Interp *interp,
+	     char *str));
+extern int Tm_CreateTmInfo _ANSI_ARGS_ (( Tcl_Interp *interp,
+    	     Widget w, char *name, char *parent, Tm_Display *displayInfo));
+extern void Tm_CreateTmInfoInChildren _ANSI_ARGS_ (( Tcl_Interp *interp,
+    	     Widget w, char *name, Tm_Display *displayInfo));
+
 
 
 /* Converter functions */
@@ -138,6 +176,14 @@ extern Boolean Tm_ConvertValueFromStringQuark _ANSI_ARGS_((Widget w,
               XtResourceList resources, int num_resources,
               char * resource, char *orig_value, XtArgVal *new_value));
 
+/* external widgets stuff */
+
+extern void Tm_ExternWidgetsInitialise _ANSI_ARGS_ ((Tcl_Interp *interp));
+extern char *Tm_ExternExpandPercent _ANSI_ARGS_ ((char *pathName,
+    		Widget w, XEvent *event, XtPointer call_data,
+    		char *value, int length));
+
+
 #ifdef __cplusplus
 }
 #endif
