@@ -166,6 +166,7 @@ Tm_ParentPath(path)
     return parent;
 }
 
+#if 0
 /*
  *--------------------------------------------------------------
  *
@@ -332,6 +333,7 @@ Tm_MakeXEvent(w, interp, xev, argc, argv)
     sprintf(interp->result, "illegal type %s\n", type);
     return TCL_ERROR;;
 }
+#endif
 
 /*
  *--------------------------------------------------------------
@@ -453,3 +455,167 @@ Tm_ParseAction(orig, action, params, num_params)
     *num_params = n;
     return TCL_OK;
 }
+
+/*
+ *--------------------------------------------------------------
+ *
+ * Tm_ClassToTmCmd --
+ *
+ *	map from a widget's class to its command info struct
+ *
+ *--------------------------------------------------------------
+ */
+Tm_Cmd *
+Tm_ClassToWidgetCommandProc(clas)
+    WidgetClass clas;
+{
+    register Tm_Cmd *cmdPtr;
+    char *command = NULL;
+    int n;
+
+    /* first find the command name from the command <-> class arrays */
+
+    /* check if it is one of the tclMotif commands */
+    n = 0;
+    while (Tm_CommToClass[n].command != NULL) {
+        if ( *(Tm_CommToClass[n].clas) ==  clas) {
+            command = Tm_CommToClass[n].command;
+            break;
+        }
+        n++;
+    }
+
+    /* check if it is one of the external widgets */
+    if (command == NULL) {
+        n = 0;
+        while (Tm_ExternCommandToClass[n].command != NULL) {
+            if ( *(Tm_ExternCommandToClass[n].clas) == clas) {
+                command = Tm_ExternCommandToClass[n].command;
+                break;
+            }
+            n++;
+        }
+    }
+    if (command == NULL) {
+	return NULL;
+    }
+
+    /* now find the command info from the Tm_Cmd arrays */
+    for (cmdPtr = Tm_Commands; cmdPtr->name != NULL; cmdPtr++) {
+	if (strcmp(command, cmdPtr->name) == 0) {
+	    return cmdPtr;
+	}
+    }
+
+    for (cmdPtr = Tm_ExternCommands; cmdPtr->name != NULL; cmdPtr++) {
+	if (strcmp(command, cmdPtr->name) == 0) {
+	    return cmdPtr;
+	}
+    }
+
+    return NULL;
+}
+
+/*
+ *--------------------------------------------------------------
+ *
+ * Tm_CreateTmInfo --
+ *
+ *	create a Tm_Widget record and place it in the widget
+ *
+ *--------------------------------------------------------------
+ */
+
+int
+Tm_CreateTmInfo(interp, w, path, parent, displayInfo)
+    Tcl_Interp *interp;
+    Widget w;
+    char *path;
+    char *parent;
+    Tm_Display *displayInfo;
+{
+    Tm_Widget *wPtr;
+    WidgetClass clas;
+    Tm_Cmd *cmdPtr;
+
+    wPtr = (Tm_Widget *) XtMalloc (sizeof (Tm_Widget));
+    wPtr -> interp = interp;
+    wPtr -> widget = w;
+    wPtr -> pathName = XtNewString(path);
+    wPtr -> parent = parent;
+    wPtr -> dropProc = NULL;
+    wPtr -> displayInfo = displayInfo;
+
+    Tm_StoreWidgetInfo(path, wPtr, interp);
+    XtVaSetValues(w, XmNuserData, wPtr, NULL);
+
+    /* create a different command for each widget type
+     */
+
+    cmdPtr = Tm_ClassToWidgetCommandProc(XtClass(w));
+    if (cmdPtr != NULL) {
+	Tcl_CreateCommand (interp, path, cmdPtr->widgetCmdProc,
+                 (ClientData) wPtr, (void (*) ()) NULL);
+    } else {
+        Tcl_CreateCommand (interp, path, Tm_AnyWidgetCmd,
+                 (ClientData) wPtr, (void (*) ()) NULL);
+    }
+
+    XtAddCallback(w, XmNdestroyCallback, Tm_DestroyWidgetHandler,
+        (XtPointer) wPtr);
+
+   Tcl_SetResult(interp, wPtr -> pathName, TCL_VOLATILE);
+
+    return TCL_OK;
+}
+
+
+/*
+ *--------------------------------------------------------------
+ *
+ * Tm_CreateTmInfoInChildren --
+ *
+ *      create a Tm_Widget record in each child of the widget
+ *
+ *--------------------------------------------------------------
+ */
+
+void 
+Tm_CreateTmInfoInChildren(interp, w, path, displayInfo)
+    Tcl_Interp *interp;
+    Widget w;
+    char *path;
+    Tm_Display *displayInfo;
+{
+    Widget *children;
+    Cardinal num_children;
+    Cardinal n;
+    char *name;
+    Widget child;
+    char *child_path;
+
+    XtVaGetValues(w, XmNchildren, &children, 
+		     XmNnumChildren, &num_children,
+		     NULL);
+
+    for (n = 0; n < num_children; n++) {
+	child = children[n];
+	name = XtName(child);
+	if (strlen(name) == 0) {
+	    /* are there any of these? be safe */
+	    continue;
+	}
+	child_path = XtMalloc(strlen(path) + strlen(name) + 2);
+	strcpy(child_path, path);
+	strcat(child_path, ".");
+	strcat(child_path, name);
+	Tm_CreateTmInfo(interp, child, child_path, path, displayInfo);
+	
+	/* and recurse down tree */
+	if (XtIsComposite(child)) {
+	    Tm_CreateTmInfoInChildren(interp, child, child_path,
+			displayInfo);
+	}
+	XtFree(child_path);
+    }
+}
