@@ -4,5 +4,5 @@
 
 xmSelectionBox .sb managed \
     -selectionLabelString "Enter text" \
-    -listItems "item 1, item 2, item 3" \
+    -listItems "{item 1} {item 2} {item 3}" \
     -listItemCount 3
