@@ -5,6 +5,6 @@
 xmPushButton .btn managed
 xmSelectionDialog .btn.bbd managed \
     -selectionLabelString "Enter or select colour" \
-    -listItems "red, green, blue" \
+    -listItems "red green blue" \
     -listItemCount 3 \
     -listLabelString "Colours:"
